@@ -51,14 +51,14 @@ size_t read_fastq_reads(fastq_vec& dst, joined_line_readers& reader,
     } catch (const fastq_error& error) {
         print_locker lock;
         std::cerr << "Error reading FASTQ record at line "
-                  << offset + dst.size()
+                  << offset + dst.size() * 4
                   << "; aborting:\n"
                   << cli_formatter::fmt(error.what()) << std::endl;
 
         throw thread_abort();
     }
 
-    return dst.size();
+    return dst.size() * 4;
 }
 
 
@@ -677,7 +677,7 @@ chunk_vec write_fastq::process(analytical_chunk* chunk)
     m_eof = file_chunk->eof;
     if (file_chunk->buffers.empty()) {
         for (const auto& line : lines) {
-            m_output << line;
+            m_output.write(line.data(), line.size());
         }
     } else {
         buffer_vec& buffers = file_chunk->buffers;
