@@ -72,6 +72,30 @@ fastq_encoding_ptr select_encoding(const std::string& name,
 }
 
 
+std::pair<unsigned, unsigned> parse_trim_argument(const string_vec& values)
+{
+    unsigned mate_1 = 0;
+    unsigned mate_2 = 0;
+
+    switch (values.size()) {
+        case 1:
+            mate_1 = str_to_unsigned(values.front());
+            mate_2 = mate_1;
+            break;
+
+        case 2:
+            mate_1 = str_to_unsigned(values.front());
+            mate_2 = str_to_unsigned(values.back());
+            break;
+
+        default:
+            throw std::invalid_argument("please specify exactly one or two values");
+    }
+
+    return std::pair<unsigned, unsigned>(mate_1, mate_2);
+}
+
+
 userconfig::userconfig(const std::string& name,
                        const std::string& version,
                        const std::string& help)
@@ -91,12 +115,15 @@ userconfig::userconfig(const std::string& name,
     , mismatch_threshold(-1.0)
     , quality_input_fmt()
     , quality_output_fmt()
+    , trim_fixed_5p(0, 0)
+    , trim_fixed_3p(0, 0)
     , trim_by_quality(false)
     , trim_window_length(std::numeric_limits<double>::quiet_NaN())
     , low_quality_score(2)
     , trim_ambiguous_bases(false)
     , max_ambiguous_bases(1000)
     , collapse(false)
+    , deterministic(false)
     , shift(2)
     , seed(get_seed())
     , max_threads(1)
@@ -120,7 +147,10 @@ userconfig::userconfig(const std::string& name,
     , interleaved(false)
     , identify_adapters(false)
     , demultiplex_sequences(false)
+    , trim5p()
+    , trim3p()
 {
+    argparser.add_header("OPTIONS:");
     argparser["--file1"] =
         new argparse::many(&input_files_1, "FILE [FILE ...]",
             "Input files containing mate 1 reads or single-ended reads; "
@@ -130,11 +160,19 @@ userconfig::userconfig(const std::string& name,
             "Input files containing mate 2 reads; if used, then the same "
             "number of files as --file1 must be listed [OPTIONAL].");
 
+    argparser["--identify-adapters"] =
+        new argparse::flag(&identify_adapters,
+            "Attempt to identify the adapter pair of PE reads, by searching "
+            "for overlapping mate reads [default: %default].");
+    argparser["--threads"] =
+        new argparse::knob(&max_threads, "THREADS",
+            "Maximum number of threads [default: %default]");
+
     argparser.add_header("FASTQ OPTIONS:");
     argparser["--qualitybase"] =
         new argparse::any(&quality_input_base, "BASE",
             "Quality base used to encode Phred scores in input; either 33, "
-            "64, or solexa [current: %default].");
+            "64, or solexa [default: %default].");
     argparser["--qualitybase-output"] =
         new argparse::any(&quality_output_base, "BASE",
             "Quality base used to encode Phred scores in output; either 33, "
@@ -155,33 +193,31 @@ userconfig::userconfig(const std::string& name,
     argparser["--interleaved"] =
         new argparse::flag(&interleaved,
             "This option enables both the --interleaved-input option and the "
-            "--interleaved-output option [current: %default].");
+            "--interleaved-output option [default: %default].");
     argparser["--interleaved-input"] =
         new argparse::flag(&interleaved_input,
             "The (single) input file provided contains both the mate 1 and "
             "mate 2 reads, one pair after the other, with one mate 1 reads "
             "followed by one mate 2 read. This option is implied by the "
-            "--interleaved option [current: %default].");
+            "--interleaved option [default: %default].");
     argparser["--interleaved-output"] =
         new argparse::flag(&interleaved_output,
             "If set, trimmed paired-end reads are written to a single file "
             "containing mate 1 and mate 2 reads, one pair after the other. "
-            "This option is implied by the --interleaved option [current: "
+            "This option is implied by the --interleaved option [default: "
             "%default].");
     argparser["--combined-output"] =
         new argparse::flag(&combined_output,
             "If set, all reads are written to the same file(s), specified by "
             "--output1 and --output2 (--output1 only if --interleaved-output "
-            "is not set). Each read is further marked by either a \"PASSED\" "
-            "or a \"FAILED\" flag, and any read that has been FAILED "
-            "(including the mate for collapsed reads) are replaced with a "
-            "single 'N' with Phred score 0 [current: %default].");
+            "is not set). Discarded reads are replaced with a single 'N' with "
+            "Phred score 0 [default: %default].");
 
     argparser.add_header("OUTPUT FILES:");
     argparser["--basename"] =
         new argparse::any(&basename, "BASENAME",
             "Default prefix for all output files for which no filename was "
-            "explicitly set [current: %default].");
+            "explicitly set [default: %default].");
     argparser["--settings"] =
         new argparse::any(nullptr, "FILE",
             "Output file containing information on the parameters used in the "
@@ -222,17 +258,17 @@ userconfig::userconfig(const std::string& name,
     argparser.add_header("OUTPUT COMPRESSION:");
     argparser["--gzip"] =
         new argparse::flag(&gzip,
-            "Enable gzip compression [current: %default]");
+            "Enable gzip compression [default: %default]");
     argparser["--gzip-level"] =
         new argparse::knob(&gzip_level, "LEVEL",
-            "Compression level, 0 - 9 [current: %default]");
+            "Compression level, 0 - 9 [default: %default]");
 
     argparser["--bzip2"] =
         new argparse::flag(&bzip2,
-            "Enable bzip2 compression [current: %default]");
+            "Enable bzip2 compression [default: %default]");
     argparser["--bzip2-level"] =
         new argparse::knob(&bzip2_level, "LEVEL",
-            "Compression level, 0 - 9 [current: %default]");
+            "Compression level, 0 - 9 [default: %default]");
 
     argparser.add_header("TRIMMING SETTINGS:");
     // Backwards compatibility with AdapterRemoval v1; not recommended due to
@@ -245,43 +281,63 @@ userconfig::userconfig(const std::string& name,
     argparser["--adapter1"] =
         new argparse::any(&adapter_1, "SEQUENCE",
             "Adapter sequence expected to be found in mate 1 reads "
-            "[current: %default].");
+            "[default: %default].");
     argparser["--adapter2"] =
         new argparse::any(&adapter_2, "SEQUENCE",
             "Adapter sequence expected to be found in mate 2 reads "
-            "[current: %default].");
+            "[default: %default].");
     argparser["--adapter-list"] =
         new argparse::any(&adapter_list, "FILENAME",
             "Read table of white-space separated adapters pairs, used as if "
             "the first column was supplied to --adapter1, and the second "
             "column was supplied to --adapter2; only the first adapter in "
-            "each pair is required SE trimming mode [current: %default].");
+            "each pair is required SE trimming mode [default: %default].");
 
     argparser.add_seperator();
+    argparser["--minadapteroverlap"] =
+        new argparse::knob(&min_adapter_overlap, "LENGTH",
+            "In single-end mode, reads are only trimmed if the overlap "
+            "between read and the adapter is at least X bases long, not "
+            "counting ambiguous nucleotides (N); this is independent of the "
+            "--minalignmentlength when using --collapse, allowing a "
+            "conservative selection of putative complete inserts while "
+            "ensuring that all possible adapter contamination is trimmed "
+            "[default: %default].");
     argparser["--mm"]
         = new argparse::floaty_knob(&mismatch_threshold, "MISMATCH_RATE",
             "Max error-rate when aligning reads and/or adapters. If > 1, the "
             "max error-rate is set to 1 / MISMATCH_RATE; if < 0, the defaults "
             "are used, otherwise the user-supplied value is used directly "
-            "[defaults: 1/3 for trimming; 1/10 when identifying adapters].");
-    argparser["--maxns"] =
-        new argparse::knob(&max_ambiguous_bases, "MAX",
-            "Reads containing more ambiguous bases (N) than this number after "
-            "trimming are discarded [current: %default].");
+            "[default: 1/3 for trimming; 1/10 when identifying adapters].");
     argparser["--shift"] =
         new argparse::knob(&shift, "N",
             "Consider alignments where up to N nucleotides are missing from "
-            "the 5' termini [current: %default].");
+            "the 5' termini [default: %default].");
 
     argparser.add_seperator();
+    argparser["--trim5p"] =
+        new argparse::many(&trim5p, "N [N]",
+            "Trim the 5' of reads by a fixed amount after removing adapters, "
+            "but before carrying out quality based trimming. Specify one "
+            "value to trim mate 1 and mate 2 reads the same amount, or two "
+            "values separated by a space to trim each mate different amounts "
+            "[default: no trimming].");
+    argparser["--trim3p"] =
+        new argparse::many(&trim3p, "N [N]",
+            "Trim the 3' of reads by a fixed amount. See --trim5p.");
+
     argparser["--trimns"] =
         new argparse::flag(&trim_ambiguous_bases,
             "If set, trim ambiguous bases (N) at 5'/3' termini "
-            "[current: %default]");
+            "[default: %default]");
+    argparser["--maxns"] =
+        new argparse::knob(&max_ambiguous_bases, "MAX",
+            "Reads containing more ambiguous bases (N) than this number after "
+            "trimming are discarded [default: %default].");
     argparser["--trimqualities"] =
         new argparse::flag(&trim_by_quality,
             "If set, trim bases at 5'/3' termini with quality scores <= to "
-            "--minquality value [current: %default]");
+            "--minquality value [default: %default]");
     argparser["--trimwindows"] =
         new argparse::floaty_knob(&trim_window_length, "INT",
             "If set, quality trimming will be carried out using window based "
@@ -291,20 +347,21 @@ userconfig::userconfig(const std::string& name,
             "multiplied with the read length to determine a window size per "
             "read. If the resulting window size is 0 or larger than the read "
             "length, the read length is used as the window size. This option "
-            "implies --trimqualities [current: %default].");
+            "implies --trimqualities [default: %default].");
     argparser["--minquality"] =
         new argparse::knob(&low_quality_score, "PHRED",
             "Inclusive minimum; see --trimqualities for details "
-            "[current: %default]");
-
+            "[default: %default]");
     argparser["--minlength"] =
         new argparse::knob(&min_genomic_length, "LENGTH",
             "Reads shorter than this length are discarded "
-            "following trimming [current: %default].");
+            "following trimming [default: %default].");
     argparser["--maxlength"] =
         new argparse::knob(&max_genomic_length, "LENGTH",
             "Reads longer than this length are discarded "
-            "following trimming [current: %default].");
+            "following trimming [default: %default].");
+
+    argparser.add_header("READ MERGING:");
     argparser["--collapse"] =
         new argparse::flag(&collapse,
             "When set, paired ended read alignments of --minalignmentlength "
@@ -315,22 +372,26 @@ userconfig::userconfig(const std::string& name,
             "reads, putative complete inserts are identified as having at "
             "least --minalignmentlength bases overlap with the adapter "
             "sequence, and are written to the the same files "
-            "[current: %default].");
+            "[default: %default].");
+    argparser["--collapse-deterministic"] =
+        new argparse::flag(&deterministic,
+            "In standard --collapse mode, AdapterRemoval will randomly select "
+            "one of two different overlapping bases if these have the same "
+            "quality (otherwise it picks the highest quality base). With "
+            "--collapse-deterministic, AdapterRemoval will instead set such "
+            "bases to N. This option implies --collapse [default: %default].");
     argparser["--minalignmentlength"] =
         new argparse::knob(&min_alignment_length, "LENGTH",
             "If --collapse is set, paired reads must overlap at least this "
             "number of bases to be collapsed, and single-ended reads must "
             "overlap at least this number of bases with the adapter to be "
-            "considered complete template molecules [current: %default].");
-    argparser["--minadapteroverlap"] =
-        new argparse::knob(&min_adapter_overlap, "LENGTH",
-            "In single-end mode, reads are only trimmed if the overlap "
-            "between read and the adapter is at least X bases long, not "
-            "counting ambiguous nucleotides (N); this is independent of the "
-            "--minalignmentlength when using --collapse, allowing a "
-            "conservative selection of putative complete inserts while "
-            "ensuring that all possible adapter contamination is trimmed "
-            "[current: %default].");
+            "considered complete template molecules [default: %default].");
+    argparser["--seed"] =
+        new argparse::knob(&seed, "SEED",
+            "Sets the RNG seed used when choosing between bases with equal "
+            "Phred scores when --collapse is enabled. This option is not "
+            "available if more than one thread is used. If not specified, a"
+            "seed is generated using the current time.");
 
     argparser.add_header("DEMULTIPLEXING:");
     argparser["--barcode-list"] =
@@ -339,7 +400,7 @@ userconfig::userconfig(const std::string& name,
             "demultiplexing. Note that both indexes should be specified for "
             "both single-end and paired-end trimming, if double-indexed "
             "multiplexing was used, in order to ensure that the demultiplexed "
-            "reads can be trimmed correctly [current: %default].");
+            "reads can be trimmed correctly [default: %default].");
     argparser["--barcode-mm"] =
         new argparse::knob(&barcode_mm, "N",
             "Maximum number of mismatches allowed when counting mismatches in "
@@ -357,23 +418,7 @@ userconfig::userconfig(const std::string& name,
     argparser["--demultiplex-only"] =
         new argparse::flag(&demultiplex_sequences,
             "Only carry out demultiplexing using the list of barcodes "
-            "supplied with --barcode-list; do not attempt to trim adapters "
-            "to carry out other processing.");
-
-    argparser.add_header("MISC:");
-    argparser["--identify-adapters"] =
-        new argparse::flag(&identify_adapters,
-            "Attempt to identify the adapter pair of PE reads, by searching "
-            "for overlapping reads [current: %default].");
-    argparser["--seed"] =
-        new argparse::knob(&seed, "SEED",
-            "Sets the RNG seed used when choosing between bases with equal "
-            "Phred scores when collapsing. Note that runs are not "
-            "deterministic if more than one thread is used. If not specified, "
-            "a seed is generated using the current time.");
-    argparser["--threads"] =
-        new argparse::knob(&max_threads, "THREADS",
-            "Maximum number of threads [current: %default]");
+            "supplied with --barcode-list. No other processing is done.");
 }
 
 
@@ -389,6 +434,9 @@ argparse::parse_result userconfig::parse_args(int argc, char *argv[])
         return result;
     }
 
+    // --collapse-deterministic implies --collapse
+    collapse |= deterministic;
+
     quality_input_fmt = select_encoding("--qualitybase", quality_input_base, quality_max);
     if (!quality_input_fmt.get()) {
         return argparse::parse_result::error;
@@ -546,6 +594,30 @@ argparse::parse_result userconfig::parse_args(int argc, char *argv[])
                   << std::endl;
     }
 
+    try {
+        if (argparser.is_set("--trim5p")) {
+            trim_fixed_5p = parse_trim_argument(trim5p);
+        }
+    } catch (const std::invalid_argument& error) {
+        std::cerr << "Error: Could not parse --trim5p argument(s): "
+                  << error.what()
+                  << std::endl;
+
+        return argparse::parse_result::error;
+    }
+
+    try {
+        if (argparser.is_set("--trim3p")) {
+            trim_fixed_3p = parse_trim_argument(trim3p);
+        }
+    } catch (const std::invalid_argument& error) {
+        std::cerr << "Error: Could not parse --trim3p argument(s): "
+                  << error.what()
+                  << std::endl;
+
+        return argparse::parse_result::error;
+    }
+
     return argparse::parse_result::ok;
 }
 
@@ -701,23 +773,6 @@ std::string userconfig::get_output_filename(const std::string& key,
 }
 
 
-fastq::ntrimmed userconfig::trim_sequence_by_quality_if_enabled(fastq& read) const
-{
-    if (trim_window_length >= 0) {
-        return read.trim_windowed_bases(trim_ambiguous_bases,
-                                        low_quality_score,
-                                        trim_window_length);
-    } else if (trim_ambiguous_bases || trim_by_quality) {
-        const char quality_score = trim_by_quality ? low_quality_score : -1;
-
-        return read.trim_trailing_bases(trim_ambiguous_bases,
-                                        quality_score);
-    }
-
-    return fastq::ntrimmed();
-}
-
-
 bool check_and_set_barcode_mm(const argparse::parser& argparser,
                               const std::string& key,
                               unsigned barcode_mm,
